import pandas as pd
import math
import csv

def calculate_metrics(data):
    # Initialize counters
    TP = TN = FP = FN = path_unk = neutral_unk = 0

    # Iterate through data to calculate TP, TN, FP, FN, path_unk, and neutral_unk
    for index, row in data.iterrows():
        label = row['labels']
        prediction = row['VEST4_pred']

        if prediction not in ['P', 'B']:
            if label == 'P':
                path_unk += 1
            elif label == 'B':
                neutral_unk += 1
        else:
            if label == 'P' and prediction == 'P':
                TP += 1
            elif label == 'P' and prediction == 'B':
                FN += 1
            elif label == 'B' and prediction == 'B':
                TN += 1
            elif label == 'B' and prediction == 'P':
                FP += 1

    # Calculate performance metrics
    accuracy = round((TP + TN) / (TP + TN + FP + FN), 3) if (TP + TN + FP + FN) > 0 else 0
    mcc = round(((TP * TN - FP * FN) / math.sqrt((TP + FP) * (TP + FN) * (TN + FP) * (TN + FN))), 3) if (TP + FP) * (
                TP + FN) * (TN + FP) * (TN + FN) > 0 else 0
    ppv = round(TP / (TP + FP), 3) if (TP + FP) > 0 else 0
    npv = round(TN / (TN + FN), 3) if (TN + FN) > 0 else 0
    sen = round(TP / (TP + FN), 3) if (TP + FN) > 0 else 0
    spec = round(TN / (FP + TN), 3) if (FP + TN) > 0 else 0
    nMCC = round((1 + mcc) / 2, 3)
    opm = round((ppv + npv) * (sen + spec) * (accuracy + nMCC) / 8, 3)

    # Total classified and coverage
    classified = TP + TN + FP + FN
    total = TP + TN + FP + FN + path_unk + neutral_unk
    coverage = round((classified / total), 3) if total > 0 else 0

    return TP, TN, FP, FN, path_unk, neutral_unk, accuracy, mcc, nMCC, ppv, npv, sen, spec, opm, coverage

def calculate_normalized_metrics(TP, TN, FP, FN, path_unk, neutral_unk):
    nominator = TN + FP
    denom = TP + FN
    up_tp = round((nominator / denom) * TP)
    up_fn = round((nominator / denom) * FN)

    norm_accuracy = round((up_tp + TN) / (up_tp + TN + FP + up_fn), 3)
    norm_mcc = round(((up_tp * TN - FP * up_fn) / math.sqrt((up_tp + FP) * (up_tp + up_fn) * (TN + FP) * (TN + up_fn))), 3)
    norm_ppv = round(up_tp / (up_tp + FP), 3)
    norm_npv = round(TN / (TN + up_fn), 3)
    norm_sen = round(up_tp / (up_tp + up_fn), 3)
    norm_spec = round(TN / (FP + TN), 3)
    norm_nMCC = round((1 + norm_mcc) / 2, 3)
    norm_opm = round((norm_ppv + norm_npv) * (norm_sen + norm_spec) * (norm_accuracy + norm_nMCC) / 8, 3)

    # Total classified and coverage
    norm_classified = up_tp + TN + FP + up_fn
    norm_total = up_tp + TN + FP + up_fn + path_unk + neutral_unk
    norm_coverage = round((norm_classified / norm_total), 3) if norm_total > 0 else 0

    return norm_accuracy, norm_mcc, norm_nMCC, norm_ppv, norm_npv, norm_sen, norm_spec, norm_opm, norm_coverage, up_tp, up_fn

def write_metrics_to_csv(output_file, metrics, norm_metrics):
    with open(output_file, 'w', newline='') as csvfile:
        csv_writer = csv.writer(csvfile)
        csv_writer.writerow(["Metric", "Value"])
        csv_writer.writerow(["Accuracy", f"{metrics['accuracy']}({norm_metrics['norm_accuracy']})"])
        csv_writer.writerow(["MCC", f"{metrics['mcc']}({norm_metrics['norm_mcc']})"])
        csv_writer.writerow(["nMCC", f"{metrics['nMCC']}({norm_metrics['norm_nMCC']})"])
        csv_writer.writerow(["PPV", f"{metrics['ppv']}({norm_metrics['norm_ppv']})"])
        csv_writer.writerow(["NPV", f"{metrics['npv']}({norm_metrics['norm_npv']})"])
        csv_writer.writerow(["Sensitivity", f"{metrics['sen']}({norm_metrics['norm_sen']})"])
        csv_writer.writerow(["Specificity", f"{metrics['spec']}({norm_metrics['norm_spec']})"])
        csv_writer.writerow(["OPM", f"{metrics['opm']}({norm_metrics['norm_opm']})"])
        csv_writer.writerow(["Pathogenic as unknown", metrics['path_unk']])
        csv_writer.writerow(["Neutral as unknown", metrics['neutral_unk']])
        csv_writer.writerow(["True Positives (TP)", f"{metrics['TP']}({norm_metrics['up_tp']})"])
        csv_writer.writerow(["True Negatives (TN)", metrics['TN']])
        csv_writer.writerow(["False Positives (FP)", metrics['FP']])
        csv_writer.writerow(["False Negatives (FN)", f"{metrics['FN']}({norm_metrics['up_fn']})"])
        csv_writer.writerow(["Coverage", f"{metrics['coverage']}({norm_metrics['norm_coverage']})"])

def main():
    # Read data from CSV file
    data = pd.read_csv('16_VEST4_pred_final.csv')

    # Calculate initial metrics
    TP, TN, FP, FN, path_unk, neutral_unk, accuracy, mcc, nMCC, ppv, npv, sen, spec, opm, coverage = calculate_metrics(data)

    # Calculate normalized metrics
    norm_accuracy, norm_mcc, norm_nMCC, norm_ppv, norm_npv, norm_sen, norm_spec, norm_opm, norm_coverage, up_tp, up_fn = calculate_normalized_metrics(TP, TN, FP, FN, path_unk, neutral_unk)

    # Combine metrics for output
    metrics = {
        'TP': TP, 'TN': TN, 'FP': FP, 'FN': FN, 'path_unk': path_unk, 'neutral_unk': neutral_unk,
        'accuracy': accuracy, 'mcc': mcc, 'nMCC': nMCC, 'ppv': ppv, 'npv': npv, 'sen': sen, 'spec': spec, 'opm': opm, 'coverage': coverage
    }
    norm_metrics = {
        'norm_accuracy': norm_accuracy, 'norm_mcc': norm_mcc, 'norm_nMCC': norm_nMCC, 'norm_ppv': norm_ppv, 'norm_npv': norm_npv,
        'norm_sen': norm_sen, 'norm_spec': norm_spec, 'norm_opm': norm_opm, 'norm_coverage': norm_coverage, 'up_tp': up_tp, 'up_fn': up_fn
    }

    # Define the output file path
    output_file = '16_VEST4_performance_metrics_normalized.csv'

    # Write metrics to CSV file
    write_metrics_to_csv(output_file, metrics, norm_metrics)

    print(f"Performance metrics have been written to {output_file}.")

if __name__ == "__main__":
    main()
